@@ -27,6 +27,9 @@
  *  2 are needed as negative exponents are handled via divide. Entries
  *  1023-1076 are required to handle denormal doubles (-1022 - 54)
 */
+#ifndef PVECLIB_DISABLE_DFP
+// clang does not support Decimal Floating Point at this time.
+
 const _Decimal128 decpowof2 [] = {
 	1.0E+0DL,	/* 2**0 */
 	2.0E+0DL,	/* 2**1 */
@@ -1109,3 +1112,4 @@ const _Decimal128 decpowof2 [] = {
 };
 
 //weak_alias(decpowof2, __decpowof2)
+#endif
