@@ -230,18 +230,25 @@ typedef __vector __bool int vb32_t;
 typedef __vector __bool long long vb64_t;
 
 /* did not get vector __int128 until GCC4.8.  */
-#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 8)
+#ifndef PVECLIB_DISABLE_INT128
+/*! \brief vector of one 128-bit signed __int128 element. */
 typedef __vector __int128 vi128_t;
+/*! \brief vector of one 128-bit unsigned __int128 element. */
 typedef __vector unsigned __int128 vui128_t;
-/*! \brief vector of 128-bit bool __int128 elements. */
+#ifndef PVECLIB_DISABLE_BOOLINT128
+/*! \brief vector of one 128-bit bool __int128 element. */
 typedef __vector __bool __int128 vb128_t;
 #else
+/*! \brief vector of one 128-bit bool __int128 element. */
+typedef __vector __bool int vb128_t;
+#endif
+#else
 /*! \brief vector of one 128-bit signed __int128 element. */
 typedef __vector int vi128_t;
 /*! \brief vector of one 128-bit unsigned __int128 element. */
 typedef __vector unsigned int vui128_t;
-/*! \brief vector of 128-bit bool __int128 elements. */
-typedef __vector __bool unsigned int vb128_t;
+/*! \brief vector of one 128-bit bool __int128 element. */
+typedef __vector __bool int vb128_t;
 #endif
 
 /*! \brief Union used to transfer 128-bit data between vector and
@@ -252,8 +259,10 @@ typedef union
   unsigned __int128 i128;
   /*! \brief Unsigned 128-bit integer from pair of 64-bit GPRs.  */
   unsigned __int128 ui128;
+#ifndef PVECLIB_DISABLE_DFP
   /*! \brief 128 bit Decimal Float from pair of double float registers.  */
   _Decimal128 dpd128;
+#endif
   /*! \brief IBM long double float from pair of double float registers.  */
   long double ldbl128;
   /*! \brief 128 bit Vector of 16 unsigned char elements.  */
@@ -382,4 +391,14 @@ typedef union
 #define VEC_BYTE_HHW 1
 #endif
 
+/*! \brief table powers of 10 [0-38] in vector __int128 format.  */
+extern const vui128_t vtipowof10[];
+/*! \brief table used to verify 128-bit frexp operations for powers of 10.  */
+extern const vui128_t vtifrexpof10[];
+
+#ifndef PVECLIB_DISABLE_DFP
+/*! \brief table powers of 2 [0-1077] in _Decimal128 format.  */
+extern const _Decimal128 decpowof2[];
+#endif
+
 #endif /* VEC_COMMON_PPC_H_ */
