@@ -258,7 +258,7 @@ vec_all_isfinitef32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x70;\n"
       : "=wa" (tmp)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
   return vec_all_eq(tmp, vec_zero);
@@ -293,7 +293,7 @@ vec_all_isinff32 (vf32_t vf32)
 {
   vui32_t tmp;
 
-#if _ARCH_PWR9
+#if _ARCH_PWR9 && !(defined(__clang__) && __clang_major__ < 9)
   const vui32_t vec_ones = CONST_VINT128_W(-1, -1, -1, -1);
 #ifdef vec_test_data_class
   tmp = (vui32_t)vec_test_data_class (vf32, 0x30);
@@ -301,7 +301,7 @@ vec_all_isinff32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x30;\n"
       : "=wa" (tmp)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
   return vec_all_eq(tmp, vec_ones);
@@ -339,7 +339,7 @@ vec_all_isnanf32 (vf32_t vf32)
 {
   vui32_t tmp;
 
-#if _ARCH_PWR9
+#if _ARCH_PWR9 && !(defined(__clang__) && __clang_major__ < 9)
   const vui32_t vec_ones = CONST_VINT128_W(-1, -1, -1, -1);
 #ifdef vec_test_data_class
   tmp = (vui32_t)vec_test_data_class (vf32, 0x40);
@@ -347,7 +347,7 @@ vec_all_isnanf32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x40;\n"
       : "=wa" (tmp)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
   return vec_all_eq(tmp, vec_ones);
@@ -393,7 +393,7 @@ vec_all_isnormalf32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x7f;\n"
       : "=wa" (tmp)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
   return vec_all_eq(tmp, vec_zero);
@@ -437,7 +437,7 @@ vec_all_issubnormalf32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x03;\n"
       : "=wa" (tmp)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
   return vec_all_eq(tmp, vec_ones);
@@ -484,7 +484,7 @@ vec_all_iszerof32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x0c;\n"
       : "=wa" (tmp)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
   return vec_all_eq(tmp, vec_ones);
@@ -528,7 +528,7 @@ vec_any_isfinitef32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x70;\n"
       : "=wa" (tmp)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
   return vec_any_eq(tmp, vec_zero);
@@ -562,7 +562,7 @@ vec_any_isinff32 (vf32_t vf32)
 {
   vui32_t tmp;
 
-#if _ARCH_PWR9
+#if _ARCH_PWR9 && !(defined(__clang__) && __clang_major__ < 9)
   const vui32_t vec_ones = CONST_VINT128_W(-1, -1, -1, -1);
 #ifdef vec_test_data_class
   tmp = (vui32_t)vec_test_data_class (vf32, 0x30);
@@ -570,7 +570,7 @@ vec_any_isinff32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x30;\n"
       : "=wa" (tmp)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
   return vec_any_eq(tmp, vec_ones);
@@ -608,7 +608,7 @@ vec_any_isnanf32 (vf32_t vf32)
 {
   vui32_t tmp;
 
-#if _ARCH_PWR9
+#if _ARCH_PWR9 && !(defined(__clang__) && __clang_major__ < 9)
   const vui32_t vec_ones = CONST_VINT128_W(-1, -1, -1, -1);
 #ifdef vec_test_data_class
   tmp = (vui32_t)vec_test_data_class (vf32, 0x40);
@@ -616,7 +616,7 @@ vec_any_isnanf32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x40;\n"
       : "=wa" (tmp)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
   return vec_any_eq(tmp, vec_ones);
@@ -662,7 +662,7 @@ vec_any_isnormalf32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x7f;\n"
       : "=wa" (tmp)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
   return vec_any_eq(tmp, vec_zero);
@@ -708,7 +708,7 @@ vec_any_issubnormalf32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x03;\n"
       : "=wa" (tmp)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
   return vec_any_eq(tmp, vec_ones);
@@ -761,7 +761,7 @@ vec_any_iszerof32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x0c;\n"
       : "=wa" (tmp)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
   return vec_any_eq(tmp, vec_ones);
@@ -836,7 +836,7 @@ vec_isfinitef32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x70;\n"
       : "=wa" (tmp2)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
   return vec_nor (tmp2, tmp2); // vec_not
@@ -873,14 +873,14 @@ static inline vb32_t
 vec_isinff32 (vf32_t vf32)
 {
   vb32_t result;
-#if _ARCH_PWR9
+#if _ARCH_PWR9  && !(defined(__clang__) && __clang_major__ < 9)
 #ifdef vec_test_data_class
   result = vec_test_data_class (vf32, 0x30);
 #else
   __asm__(
       "xvtstdcsp %x0,%x1,0x30;\n"
       : "=wa" (result)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
 #else
@@ -915,14 +915,14 @@ static inline vb32_t
 vec_isnanf32 (vf32_t vf32)
 {
   vb32_t result;
-#if _ARCH_PWR9
+#if _ARCH_PWR9 && !(defined(__clang__) && __clang_major__ < 9)
 #ifdef vec_test_data_class
   result = vec_test_data_class (vf32, 0x40);
 #else
   __asm__(
       "xvtstdcsp %x0,%x1,0x40;\n"
       : "=wa" (result)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
 #else
@@ -968,7 +968,7 @@ vec_isnormalf32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x7f;\n"
       : "=wa" (tmp2)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
   return vec_nor (tmp2, tmp2); // vec_not
@@ -1014,7 +1014,7 @@ vec_issubnormalf32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x03;\n"
       : "=wa" (result)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
 #else
@@ -1062,7 +1062,7 @@ vec_iszerof32 (vf32_t vf32)
   __asm__(
       "xvtstdcsp %x0,%x1,0x0c;\n"
       : "=wa" (result)
-      : "wf" (vf32)
+      : "wa" (vf32)
       :);
 #endif
 #else
