@@ -79,7 +79,13 @@
  * - Commonly used operations, not covered by the ABI or
  * <altivec.h>, and require multiple instructions or
  * are not obvious.
- * Examples include the multiply high and shift immediate operations.
+ * Examples include the multiply-add, multiply-high and shift immediate
+ * operations.
+ *
+ * \section i16_recent_additions Recent Additions
+ *
+ * Added vec_vmaddeuh() and vec_vmaddouh() as an optimization for
+ * the vector multiply quadword implementations on POWER7.
  *
  * \section i16_endian_issues_0_0 Endian problems with halfword operations
  *
@@ -548,14 +554,16 @@ vec_clzh (vui16_t vra)
 {
   vui16_t r;
 #ifdef _ARCH_PWR8
-#ifndef vec_vclzh
+#if defined (vec_vclzh)
+  r = vec_vclzh (vra);
+#elif defined (__clang__)
+  r = vec_cntlz (vra);
+#else
   __asm__(
       "vclzh %0,%1;"
       : "=v" (r)
       : "v" (vra)
       : );
-#else
-  r = vec_vclzh (vra);
 #endif
 #else
 //#warning Implememention pre power8
@@ -827,14 +835,16 @@ vec_popcnth (vui16_t vra)
 {
   vui16_t r;
 #ifdef _ARCH_PWR8
-#ifndef vec_vpopcnth
+#if defined (vec_vpopcnth)
+  r = vec_vpopcnth (vra);
+#elif defined (__clang__)
+  r = vec_popcnt (vra);
+#else
   __asm__(
       "vpopcnth %0,%1;"
       : "=v" (r)
       : "v" (vra)
       : );
-#else
-  r = vec_vpopcnth (vra);
 #endif
 #else
   //#warning Implememention pre power8
@@ -899,14 +909,14 @@ vec_revbh (vui16_t vra)
   vui16_t result;
 
 #ifdef _ARCH_PWR9
-#ifndef vec_revb
+#if defined (vec_revb) || defined (__clang__)
+  result = vec_revb (vra);
+#else
   __asm__(
       "xxbrh %x0,%x1;"
       : "=wa" (result)
       : "wa" (vra)
       : );
-#else
-  result = vec_revb (vra);
 #endif
 #else
 #if __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
@@ -1062,6 +1072,78 @@ vec_srahi (vi16_t vra, const unsigned int shb)
   return (vi16_t) result;
 }
 
+/** \brief Vector Multiply-Add Even Unsigned Halfwords.
+ *
+ *  Multiply the even 16-bit Words of vector unsigned short
+ *  values (a * b) and return sums of the unsigned 32-bit product and
+ *  the even 16-bit halfwords of c
+ *  (a<SUB>even</SUB> * b<SUB>even</SUB>) + EXTZ(c<SUB>even</SUB>).
+ *
+ *  \note The advantage of this form (versus Multiply-Sum) is that
+ *  the final 32 bit sums can not overflow.
+ *  \note This implementation is NOT endian sensitive and the function is
+ *  stable across BE/LE implementations.
+ *
+ *  |processor|Latency|Throughput|
+ *  |--------:|:-----:|:---------|
+ *  |power8   | 9-18  | 2/cycle  |
+ *  |power9   | 9-16  | 2/cycle  |
+ *
+ *  @param a 128-bit vector unsigned short.
+ *  @param b 128-bit vector unsigned short.
+ *  @param c 128-bit vector unsigned short.
+ *  @return vector unsigned int sum (a<SUB>even</SUB> * b<SUB>even</SUB>) + EXTZ(c<SUB>even</SUB>).
+ */
+static inline vui32_t
+vec_vmaddeuh (vui16_t a, vui16_t b, vui16_t c)
+{
+  const vui16_t zero = { 0, 0, 0, 0,  0, 0, 0, 0 };
+  vui32_t res;
+  vui16_t c_euh = vec_mrgahh ((vui32_t) zero, (vui32_t) c);
+#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+  res = vec_vmulouh (a, b);
+#else
+  res = vec_vmuleuh (a, b);
+#endif
+  return vec_vadduwm (res, (vui32_t) c_euh);
+}
+
+/** \brief Vector Multiply-Add Odd Unsigned Halfwords.
+ *
+ *  Multiply the odd 16-bit Halfwords of vector unsigned short
+ *  values (a * b) and return sums of the unsigned 32-bit product and
+ *  the odd 16-bit halfwords of c
+ *  (a<SUB>odd</SUB> * b<SUB>odd</SUB>) + EXTZ(c<SUB>odd</SUB>).
+ *
+ *  \note The advantage of this form (versus Multiply-Sum) is that
+ *  the final 32 bit sums can not overflow.
+ *  \note This implementation is NOT endian sensitive and the function is
+ *  stable across BE/LE implementations.
+ *
+ *  |processor|Latency|Throughput|
+ *  |--------:|:-----:|:---------|
+ *  |power8   | 9-18  | 2/cycle  |
+ *  |power9   | 9-16  | 2/cycle  |
+ *
+ *  @param a 128-bit vector unsigned short.
+ *  @param b 128-bit vector unsigned short.
+ *  @param c 128-bit vector unsigned short.
+ *  @return vector unsigned int sum (a<SUB>odd</SUB> * b<SUB>odd</SUB>) + EXTZ(c<SUB>odd</SUB>).
+ */
+static inline vui32_t
+vec_vmaddouh (vui16_t a, vui16_t b, vui16_t c)
+{
+  const vui16_t zero = { 0, 0, 0, 0,  0, 0, 0, 0 };
+  vui32_t res;
+  vui16_t c_ouh = vec_mrgalh ((vui32_t) zero, (vui32_t) c);
+#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+  res = vec_vmuleuh (a, b);
+#else
+  res = vec_vmulouh (a, b);
+#endif
+  return vec_vadduwm (res, (vui32_t) c_ouh);
+}
+
 /** \brief Vector Merge Even Halfwords.
  *
  * Merge the even halfword elements from the concatenation of 2 x
