@@ -85,6 +85,12 @@
  * Examples include the shift immediate, merge algebraic high/low,
  * and multiply high operations.
  *
+ * \section i32_recent_additions Recent Additions
+ *
+ * Added vec_vmaddeuw(), vec_vmaddouw(), vec_vmadd2euw(), and
+ * vec_vmadd2ouw() as an optimization for
+ * the vector multiply quadword implementations on POWER8.
+ *
  * \section i32_endian_issues_0_0 Endian problems with word operations
  *
  * It would be useful to provide a vector multiply high word
@@ -385,6 +391,14 @@ example_convert_timebase (vui32_t *tb, vui32_t *timespec, int n)
  * \ref perf_data
  */
 
+///@cond INTERNAL
+static inline vui64_t vec_muleuw (vui32_t a, vui32_t b);
+static inline vui64_t vec_mulouw (vui32_t a, vui32_t b);
+static inline vi32_t vec_srawi (vi32_t vra, const unsigned int shb);
+static inline vui64_t vec_vmuleuw (vui32_t a, vui32_t b);
+static inline vui64_t vec_vmulouw (vui32_t a, vui32_t b);
+///@endcond
+
 /** \brief Vector Absolute Difference Unsigned Word.
  *
  *  Compute the absolute difference for each word.
@@ -452,14 +466,16 @@ vec_clzw (vui32_t vra)
 {
   vui32_t r;
 #ifdef _ARCH_PWR8
-#ifndef vec_vclzw
+#if defined (vec_vclzw)
+  r = vec_vclzw (vra);
+#elif defined (__clang__)
+  r = vec_cntlz (vra);
+#else
   __asm__(
       "vclzw %0,%1;"
       : "=v" (r)
       : "v" (vra)
       : );
-#else
-  r = vec_vclzw (vra);
 #endif
 #else
 //#warning Implememention pre POWER8
@@ -530,6 +546,9 @@ vec_clzw (vui32_t vra)
  * as generated by vec_muleuw/vec_mulouw. This result is effectively
  * a vector multiply high unsigned word.
  *
+ * \note This implementation is NOT endian sensitive and the function
+ * is stable across BE/LE implementations.
+ *
  *  |processor|Latency|Throughput|
  *  |--------:|:-----:|:---------|
  *  |power8   |   2   | 2/cycle  |
@@ -577,6 +596,9 @@ vec_mrgahw (vui64_t vra, vui64_t vrb)
  * as generated by vec_muleuw/vec_mulouw. This result is effectively a
  * vector multiply low unsigned word (multiply unsigned word modulo).
  *
+ * \note This implementation is NOT endian sensitive and the function
+ * is stable across BE/LE implementations.
+ *
  * |processor|Latency|Throughput|
  * |--------:|:-----:|:---------|
  * |power8   |   2   | 2/cycle  |
@@ -719,6 +741,16 @@ vec_mrgow (vui32_t vra, vui32_t vrb)
  * Multiple the even words of two vector signed int values and return
  * the signed long product of the even words.
  *
+ * For POWER8 and later we can use the vmulesw instruction.
+ * But for POWER7 and earlier we have to construct word multiplies
+ * from halfword multiplies. See vec_muleuw().
+ *
+ * Here we start with a unsigned vec_muleuw product, then correct the
+ * high 32-bits of the product to signed. Based on:
+ * Warren, Henry S. Jr and <I>Hacker's Delight</I>, 2nd Edition,
+ * Addison Wesley, 2013. Chapter 8 Multiplication, Section 8-3
+ * High-Order Product Signed from/to Unsigned.
+ *
  * |processor|Latency|Throughput|
  * |--------:|:-----:|:---------|
  * |power8   |   7   | 2/cycle  |
@@ -732,6 +764,8 @@ static inline vi64_t
 vec_mulesw (vi32_t a, vi32_t b)
 {
   vi64_t res;
+#ifdef _ARCH_PWR8
+  // The vector vmulosw/vmulesw instructions introduced in PRW8
 #if defined __GNUC__ && (__GNUC__ > 7)
   res = vec_mule (a, b);
 #else
@@ -745,6 +779,33 @@ vec_mulesw (vi32_t a, vi32_t b)
       : "v" (a),
       "v" (b)
       : );
+#endif
+#else
+  // must be PWR7 or older
+  vui32_t uia, uib;
+  vui32_t amask, bmask, t1, t2, r;
+  vui64_t ui_prod;
+  const vui32_t zero= { 0,0,0,0};
+
+  uia = (vui32_t) a;
+  uib = (vui32_t) b;
+  // Generate 32-bit masks from the sign of each input word.
+  amask = (vui32_t) vec_srawi (a, 31);
+  bmask = (vui32_t) vec_srawi (b, 31);
+  // Extend the even masks to the right with zeros to form two 64-bit
+  // masks. We need the trailing zeros as the low 32-bits of the
+  // product are correct as-is and should not change.
+  amask = vec_mrgew (amask, zero);
+  bmask = vec_mrgew (bmask, zero);
+  // Compute the doubleword even unsigned word product
+  ui_prod = vec_muleuw (uia, uib);
+
+  // Generate t1 = amask & b and t2 = bmask & a
+  t1 = vec_and (amask, uib);
+  t2 = vec_and (bmask, uia);
+  // Apply the correction res = ui_prod - t1 - t2
+  r = vec_sub ((vui32_t) ui_prod, t1);
+  res = (vi64_t) vec_sub (r, t2);
 #endif
   return (res);
 }
@@ -754,6 +815,16 @@ vec_mulesw (vi32_t a, vi32_t b)
  * Multiple the odd words of two vector signed int values and return
  * the signed long product of the odd words.
  *
+ * For POWER8 and later we can use the vmulosw instruction.
+ * But for POWER7 and earlier we have to construct word multiplies
+ * from halfword multiplies. See vec_mulouw().
+ *
+ * Here we start with a unsigned vec_mulouw product, then correct the
+ * high-order 32-bits of the product to signed. Based on:
+ * Warren, Henry S. Jr and <I>Hacker's Delight</I>, 2nd Edition,
+ * Addison Wesley, 2013. Chapter 8 Multiplication, Section 8-3
+ * High-Order Product Signed from/to Unsigned.
+ *
  * |processor|Latency|Throughput|
  * |--------:|:-----:|:---------|
  * |power8   |   7   | 2/cycle  |
@@ -767,7 +838,9 @@ static inline vi64_t
 vec_mulosw (vi32_t a, vi32_t b)
 {
   vi64_t res;
-#if defined __GNUC__ && (__GNUC__ > 7)
+#ifdef _ARCH_PWR8
+  // The vector vmulosw/vmulesw instructions introduced in PRW8
+#if defined __GNUC__ && (__GNUC__ > 7) && (__BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__)
   res = vec_mulo (a, b);
 #else
   __asm__(
@@ -780,6 +853,36 @@ vec_mulosw (vi32_t a, vi32_t b)
       : "v" (a),
       "v" (b)
       : );
+#endif
+#else
+  // must be PWR7 or older
+  vui32_t uia, uib;
+  vui32_t amask, bmask, t1, t2, r;
+  vui64_t ui_prod;
+  const vui32_t zero= { 0,0,0,0};
+
+  // duplicate odd words to even
+  uia = (vui32_t) a;
+  uib = (vui32_t) b;
+  uia = vec_mrgow (uia, uia);
+  uib = vec_mrgow (uib, uib);
+  // Generate 32-bit masks from the sign of each input word.
+  amask = (vui32_t) vec_srawi ((vi32_t) uia, 31);
+  bmask = (vui32_t) vec_srawi ((vi32_t) uib, 31);
+  // Shift the odd masks to the left 32 and extend to the right with
+  // zeros to form two 64-bit masks. We need the trailing zeros as the
+  // low 32-bits of the product are correct as-is.
+  amask = vec_mrgow (amask, zero);
+  bmask = vec_mrgow (bmask, zero);
+  // Compute the doubleword odd unsigned word product
+  ui_prod = vec_mulouw (uia, uib);
+
+  // Generate t1 = amask & b and t2 = bmask & a
+  t1 = vec_and (amask, uib);
+  t2 = vec_and (bmask, uia);
+  // Apply the correction res = ui_prod - t1 - t2
+  r = vec_sub ((vui32_t) ui_prod, t1);
+  res = (vi64_t) vec_sub (r, t2);
 #endif
   return (res);
 }
@@ -789,6 +892,18 @@ vec_mulosw (vi32_t a, vi32_t b)
  * Multiple the even words of two vector unsigned int values and return
  * the unsigned long product of the even words.
  *
+ * For POWER8 and later we can use the vmuleuw instruction.
+ * But for POWER7 and earlier we have to construct word multiplies
+ * from two halfword multiplies (vmuleuh and vmulouh). Then sum the
+ * partial products for the final doubleword results. This is
+ * complicated by the fact that vector add doubleword is not available
+ * for POWER7. So we need to construct the doubleword add from
+ * Vector Add Unsigned Word Modulo (vadduwm) and
+ * Vector Add and Write Carry-Out Unsigned Word (vaddcuw) with
+ * shift double quadword to reposition the low word carry and
+ * a final vadduwm to complete the carry propagation for the
+ * doubleword add.
+ *
  * |processor|Latency|Throughput|
  * |--------:|:-----:|:---------|
  * |power8   |   7   | 2/cycle  |
@@ -801,78 +916,11 @@ vec_mulosw (vi32_t a, vi32_t b)
 static inline vui64_t
 vec_muleuw (vui32_t a, vui32_t b)
 {
-  vui64_t res;
-#ifdef _ARCH_PWR8
-#if defined __GNUC__ && (__GNUC__ > 7)
-  res = vec_mule (a, b);
-#else
-  __asm__(
 #if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-      "vmulouw %0,%1,%2;\n"
+  return vec_vmulouw (a, b);
 #else
-      "vmuleuw %0,%1,%2;\n"
+  return vec_vmuleuw (a, b);
 #endif
-      : "=v" (res)
-      : "v" (a),
-      "v" (b)
-      : );
-#endif
-#else
-  const vui32_t zero = {0,0,0,0};
-  const vui32_t ones = {-1,-1,-1,-1};
-  vui32_t wmask01;
-  vui32_t p0, p1, pp10, pp01, resw;
-  vui16_t m0, m1, mt, mth, mtl;
-
-  /* generate {0,-1,0,-1}  mask.  */
-  wmask01 = vec_vmrghw (zero, ones);
-
-  mt = (vui16_t)b;
-  mtl = vec_mergeh (mt, mt);
-  mth = vec_mergel (mt, mt);
-
-#ifdef _ARCH_PWR7
-  m0 = (vui16_t)vec_xxpermdi ((vui64_t)mtl, (vui64_t)mth, 0);
-#else
-  {
-    vui32_t temp;
-    temp = vec_sld ((vui32_t) mtl, (vui32_t) mth, 8);
-    m0 = (vui16_t) vec_sld (temp, (vui32_t) mth, 8);
-  }
-#endif
-
-  resw = vec_sld (a, a, 12);
-  resw = vec_sel (a, resw, wmask01);
-  m1 = (vui16_t)resw;
-
-  p0 = vec_vmuleuh (m1, m0);
-  p1 = vec_vmulouh (m1, m0);
-  resw = vec_sel (p0, p1, wmask01);
-  res = (vui64_t)resw;
-
-  pp10 = vec_sld (p1, p1, 12);
-  pp01 = p0;
-  /* pp01 = vec_addudm (pp01, pp10).  */
-  {
-    vui32_t c;
-    vui32_t xmask;
-    xmask = vec_sld (wmask01, wmask01, 2);
-    c    = vec_vaddcuw (pp01, pp10);
-    pp01 = vec_vadduwm (pp01, pp10);
-    c    = vec_sld (c, c, 6);
-    pp01 = vec_sld (pp01, pp01, 2);
-    pp01 = vec_sel (c, pp01, xmask);
-  }
-  /* res = vec_addudm (pp01, res).  */
-  {
-    vui32_t c, r;
-    c = vec_vaddcuw (pp01, (vui32_t)res);
-    r = vec_vadduwm (pp01, (vui32_t)res);
-    c = vec_sld (c, zero, 4);
-    res = (vui64_t)vec_vadduwm (r, c);
-  }
-#endif
-  return (res);
 }
 
 /** \brief Vector multiply odd unsigned words.
@@ -880,6 +928,18 @@ vec_muleuw (vui32_t a, vui32_t b)
  * Multiple the odd words of two vector unsigned int values and return
  * the unsigned long product of the odd words.
  *
+ * For POWER8 and later we can use the vmulouw instruction.
+ * But for POWER7 and earlier we have to construct word multiplies
+ * from two halfword multiplies (vmuleuh and vmulouh). Then sum the
+ * partial products for the final doubleword results. This is
+ * complicated by the fact that vector add doubleword is not available
+ * for POWER7. So we need to construct the doubleword add from
+ * Vector Add Unsigned Word Modulo (vadduwm) and
+ * Vector Add and Write Carry-Out Unsigned Word (vaddcuw) with
+ * shift double quadword to reposition the low word carry and
+ * a final vadduwm to complete the carry propagation for the doubleword
+ * add.
+ *
  * |processor|Latency|Throughput|
  * |--------:|:-----:|:---------|
  * |power8   |   7   | 2/cycle  |
@@ -892,77 +952,11 @@ vec_muleuw (vui32_t a, vui32_t b)
 static inline vui64_t
 vec_mulouw (vui32_t a, vui32_t b)
 {
-  vui64_t res;
-#ifdef _ARCH_PWR8
-#if defined __GNUC__ && (__GNUC__ > 7)
-  /* Not supported in GCC yet.  ETA GCC-8.  */
-  res = vec_mulo (a, b);
-#else
-  __asm__(
 #if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-      "vmuleuw %0,%1,%2;\n"
+  return vec_vmuleuw (a, b);
 #else
-      "vmulouw %0,%1,%2;\n"
+  return vec_vmulouw (a, b);
 #endif
-      : "=v" (res)
-      : "v" (a),
-      "v" (b)
-      : );
-#endif
-#else
-  const vui32_t zero = {0,0,0,0};
-  const vui32_t ones = {-1,-1,-1,-1};
-  vui32_t wmask01;
-  vui32_t p0, p1, pp10, pp01, resw;
-  vui16_t m0, m1, mt, mth, mtl;
-  /* generate {0,-1,0,-1}  mask.  */
-  wmask01 = vec_vmrghw (zero, ones);
-
-  mt = (vui16_t)b;
-  mtl = vec_mergel (mt, mt);
-  mth = vec_mergeh (mt, mt);
-#ifdef _ARCH_PWR7
-  m0 = (vui16_t)vec_xxpermdi ((vui64_t)mth, (vui64_t)mtl, 3);
-#else
-  {
-    vui32_t temp;
-    temp = vec_sld ((vui32_t) mtl, (vui32_t) mtl, 8);
-    result = (vui64_t) vec_sld ((vui32_t) mth, temp, 8);
-  }
-#endif
-
-  resw = vec_sld (a, a, 4);
-  m1 = (vui16_t)vec_sel (resw, a, wmask01);
-
-  p0 = vec_vmuleuh (m1, m0);
-  p1 = vec_vmulouh (m1, m0);
-
-  resw = vec_sel (p0, p1, wmask01);
-  res = (vui64_t)resw;
-
-  pp10 = vec_sld (p1, p1, 12);
-  pp01 = p0;
-  /* pp01 = vec_addudm (pp01, pp10).  */
-  {
-    vui32_t c;
-    vui32_t xmask;
-    xmask = vec_sld (wmask01, wmask01, 2);
-    c    = vec_vaddcuw (pp01, pp10);
-    pp01 = vec_vadduwm (pp01, pp10);
-    c    = vec_sld (c, c, 6);
-    pp01 = vec_sld (pp01, pp01, 2);
-    pp01 = vec_sel (c, pp01, xmask);
-  }
-  /* res = vec_addudm (pp01, res).  */
-  {
-    vui32_t c, r;
-    c = vec_vaddcuw (pp01, (vui32_t)res);
-    r = vec_vadduwm (pp01, (vui32_t)res);
-    c = vec_sld (c, zero, 4);
-    res = (vui64_t)vec_vadduwm (r, c);
-  }
-#endif
-  return (res);
 }
 
 /** \brief Vector Multiply High Signed Word.
@@ -1101,14 +1095,16 @@ vec_popcntw (vui32_t vra)
 {
   vui32_t r;
 #ifdef _ARCH_PWR8
-#ifndef vec_vpopcntw
+#if defined (vec_vpopcntw)
+  r = vec_vpopcntw (vra);
+#elif defined (__clang__)
+  r = vec_popcnt (vra);
+#else
   __asm__(
       "vpopcntw %0,%1;"
       : "=v" (r)
       : "v" (vra)
       : );
-#else
-  r = vec_vpopcntw (vra);
 #endif
 #else
 //#warning Implememention pre power8
@@ -1145,14 +1141,14 @@ vec_revbw (vui32_t vra)
   vui32_t result;
 
 #ifdef _ARCH_PWR9
-#ifndef vec_revb
+#if defined (vec_revb) || defined (__clang__)
+  result = vec_revb (vra);
+#else
   __asm__(
       "xxbrw %x0,%x1;"
       : "=wa" (result)
       : "wa" (vra)
       : );
-#else
-  result = vec_revb (vra);
 #endif
 #else
   const vui64_t vconstp =
@@ -1308,4 +1304,257 @@ vec_srwi (vui32_t vra, const unsigned int shb)
   return (vui32_t) result;
 }
 
+/** \brief \copybrief vec_int64_ppc.h::vec_vmadd2euw()
+ *
+ * \note this implementation exists in
+ * \ref vec_int64_ppc.h::vec_vmadd2euw()
+ * as it requires vec_addudm().
+ */
+static inline vui64_t
+vec_vmadd2euw (vui32_t a, vui32_t b, vui32_t c, vui32_t d);
+
+/** \brief \copybrief vec_int64_ppc.h::vec_vmadd2ouw()
+ *
+ * \note this implementation exists in
+ * \ref vec_int64_ppc.h::vec_vmadd2ouw()
+ * as it requires vec_addudm().
+ */
+static inline vui64_t
+vec_vmadd2ouw (vui32_t a, vui32_t b, vui32_t c, vui32_t d);
+
+/** \brief \copybrief vec_int64_ppc.h::vec_vmaddeuw()
+ *
+ * \note this implementation exists in
+ * \ref vec_int64_ppc.h::vec_vmaddeuw()
+ * as it requires vec_addudm().
+ */
+static inline vui64_t
+vec_vmaddeuw (vui32_t a, vui32_t b, vui32_t c);
+
+/** \brief \copybrief vec_int64_ppc.h::vec_vmaddouw()
+ *
+ * \note this implementation exists in
+ * \ref vec_int64_ppc.h::vec_vmaddouw()
+ * as it requires vec_addudm().
+ */
+static inline vui64_t
+vec_vmaddouw (vui32_t a, vui32_t b, vui32_t c);
+
+/** \brief \copybrief vec_int64_ppc.h::vec_vmsumuwm()
+ *
+ * \note this implementation exists in
+ * \ref vec_int64_ppc.h::vec_vmsumuwm()
+ * as it requires vec_addudm().
+ */
+static inline vui64_t
+vec_vmsumuwm (vui32_t a, vui32_t b, vui64_t c);
+
+/** \brief Vector Multiply Even Unsigned words.
+ *
+ * Multiply the even words of two vector unsigned int values and return
+ * the unsigned long product of the even words.
+ *
+ * For POWER8 and later we can use the vmuleuw instruction.
+ * But for POWER7 and earlier we have to construct word multiplies
+ * from two halfword multiplies (vmuleuh and vmulouh). Then sum the
+ * partial products for the final doubleword results. This is
+ * complicated by the fact that vector add doubleword is not available
+ * for POWER7. So we need to construct the doubleword add from
+ * Vector Add Unsigned Word Modulo (vadduwm) and
+ * Vector Add and Write Carry-Out Unsigned Word (vaddcuw) with
+ * shift double quadword to reposition the low word carry and
+ * a final vadduwm to complete the carry propagation for the
+ * doubleword add.
+ *
+ * \note This implementation is NOT endian sensitive and the function
+ * is stable across BE/LE implementations.
+ *
+ * |processor|Latency|Throughput|
+ * |--------:|:-----:|:---------|
+ * |power8   |   7   | 2/cycle  |
+ * |power9   |   7   | 2/cycle  |
+ *
+ * @param vra 128-bit vector unsigned int.
+ * @param vrb 128-bit vector unsigned int.
+ * @return vector unsigned long product of the even words of a and b.
+ */
+static inline vui64_t
+vec_vmuleuw (vui32_t vra, vui32_t vrb)
+{
+  vui64_t res;
+#ifdef _ARCH_PWR8
+#if defined __GNUC__ && (__GNUC__ > 7)
+#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+  res = vec_mulo (vra, vrb);
+#else
+  res = vec_mule (vra, vrb);
+#endif
+#else
+  __asm__(
+      "vmuleuw %0,%1,%2;\n"
+      : "=v" (res)
+      : "v" (vra),
+      "v" (vrb)
+      : );
+#endif
+#else
+  const vui32_t zero = {0,0,0,0};
+  const vui32_t ones = {-1,-1,-1,-1};
+  vui32_t wmask01;
+  vui32_t p0, p1, pp10, pp01, resw;
+  vui16_t m0, m1, mt, mth, mtl;
+
+  /* generate {0,-1,0,-1}  mask.  */
+  wmask01 = vec_vmrghw (zero, ones);
+
+  mt = (vui16_t)vrb;
+  mtl = vec_mergeh (mt, mt);
+  mth = vec_mergel (mt, mt);
+
+#ifdef _ARCH_PWR7
+  m0 = (vui16_t)vec_xxpermdi ((vui64_t)mtl, (vui64_t)mth, 0);
+#else
+  {
+    vui32_t temp;
+    temp = vec_sld ((vui32_t) mtl, (vui32_t) mth, 8);
+    m0 = (vui16_t) vec_sld (temp, (vui32_t) mth, 8);
+  }
+#endif
+
+  resw = vec_sld (vra, vra, 12);
+  resw = vec_sel (vra, resw, wmask01);
+  m1 = (vui16_t)resw;
+
+  p0 = vec_vmuleuh (m1, m0);
+  p1 = vec_vmulouh (m1, m0);
+  resw = vec_sel (p0, p1, wmask01);
+  res = (vui64_t)resw;
+
+  pp10 = vec_sld (p1, p1, 12);
+  pp01 = p0;
+  /* pp01 = vec_addudm (pp01, pp10).  */
+  {
+    vui32_t c;
+    vui32_t xmask;
+    xmask = vec_sld (wmask01, wmask01, 2);
+    c    = vec_vaddcuw (pp01, pp10);
+    pp01 = vec_vadduwm (pp01, pp10);
+    c    = vec_sld (c, c, 6);
+    pp01 = vec_sld (pp01, pp01, 2);
+    pp01 = vec_sel (c, pp01, xmask);
+  }
+  /* res = vec_addudm (pp01, res).  */
+  {
+    vui32_t c, r;
+    c = vec_vaddcuw (pp01, (vui32_t)res);
+    r = vec_vadduwm (pp01, (vui32_t)res);
+    c = vec_sld (c, zero, 4);
+    res = (vui64_t)vec_vadduwm (r, c);
+  }
+#endif
+  return (res);
+}
+
+/** \brief Vector Multiply Odd Unsigned Words.
+ *
+ * Multiply the odd words of two vector unsigned int values and return
+ * the unsigned long product of the odd words.
+ *
+ * For POWER8 and later we can use the vmulouw instruction.
+ * But for POWER7 and earlier we have to construct word multiplies
+ * from two halfword multiplies (vmuleuh and vmulouh). Then sum the
+ * partial products for the final doubleword results. This is
+ * complicated by the fact that vector add doubleword is not available
+ * for POWER7. So we need to construct the doubleword add from
+ * Vector Add Unsigned Word Modulo (vadduwm) and
+ * Vector Add and Write Carry-Out Unsigned Word (vaddcuw) with
+ * shift double quadword to reposition the low word carry and
+ * a final vadduwm to complete the carry propagation for the doubleword
+ * add.
+ *
+ * |processor|Latency|Throughput|
+ * |--------:|:-----:|:---------|
+ * |power8   |   7   | 2/cycle  |
+ * |power9   |   7   | 2/cycle  |
+ *
+ * @param vra 128-bit vector unsigned int.
+ * @param vrb 128-bit vector unsigned int.
+ * @return vector unsigned long product of the odd words of a and b.
+ */
+static inline vui64_t
+vec_vmulouw (vui32_t vra, vui32_t vrb)
+{
+  vui64_t res;
+#ifdef _ARCH_PWR8
+#if defined __GNUC__ && (__GNUC__ > 7)
+  /* Not supported in GCC yet.  ETA GCC-8.  */
+#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+  res = vec_mule (vra, vrb);
+#else
+  res = vec_mulo (vra, vrb);
+#endif
+#else
+  __asm__(
+      "vmulouw %0,%1,%2;\n"
+      : "=v" (res)
+      : "v" (vra),
+      "v" (vrb)
+      : );
+#endif
+#else
+  const vui32_t zero = {0,0,0,0};
+  const vui32_t ones = {-1,-1,-1,-1};
+  vui32_t wmask01;
+  vui32_t p0, p1, pp10, pp01, resw;
+  vui16_t m0, m1, mt, mth, mtl;
+  /* generate {0,-1,0,-1}  mask.  */
+  wmask01 = vec_vmrghw (zero, ones);
+
+  mt = (vui16_t)vrb;
+  mtl = vec_mergel (mt, mt);
+  mth = vec_mergeh (mt, mt);
+#ifdef _ARCH_PWR7
+  m0 = (vui16_t)vec_xxpermdi ((vui64_t)mth, (vui64_t)mtl, 3);
+#else
+  {
+    vui32_t temp;
+    temp = vec_sld ((vui32_t) mtl, (vui32_t) mtl, 8);
+    result = (vui64_t) vec_sld ((vui32_t) mth, temp, 8);
+  }
+#endif
+
+  resw = vec_sld (vra, vra, 4);
+  m1 = (vui16_t)vec_sel (resw, vra, wmask01);
+
+  p0 = vec_vmuleuh (m1, m0);
+  p1 = vec_vmulouh (m1, m0);
+
+  resw = vec_sel (p0, p1, wmask01);
+  res = (vui64_t)resw;
+
+  pp10 = vec_sld (p1, p1, 12);
+  pp01 = p0;
+  /* pp01 = vec_addudm (pp01, pp10).  */
+  {
+    vui32_t c;
+    vui32_t xmask;
+    xmask = vec_sld (wmask01, wmask01, 2);
+    c    = vec_vaddcuw (pp01, pp10);
+    pp01 = vec_vadduwm (pp01, pp10);
+    c    = vec_sld (c, c, 6);
+    pp01 = vec_sld (pp01, pp01, 2);
+    pp01 = vec_sel (c, pp01, xmask);
+  }
+  /* res = vec_addudm (pp01, res).  */
+  {
+    vui32_t c, r;
+    c = vec_vaddcuw (pp01, (vui32_t)res);
+    r = vec_vadduwm (pp01, (vui32_t)res);
+    c = vec_sld (c, zero, 4);
+    res = (vui64_t)vec_vadduwm (r, c);
+  }
+#endif
+  return (res);
+}
+
 #endif /* VEC_INT32_PPC_H_ */
