@@ -22,15 +22,15 @@
 
 #include <stdint.h>
 #include <stdio.h>
+#include <string.h>
 #include <float.h>
 #define __DEBUG_PRINT__
 #include <pveclib/vec_f128_ppc.h>
 #include <testsuite/arith128_print.h>
-//#include "math_private.h"
 #include "ieee754.h"
 
 long tcount;
-
+#ifndef PVECLIB_DISABLE_DFP
 void
 print_dfp128 (char *prefix, _Decimal128 val128)
 {
@@ -93,7 +93,7 @@ check_dfp128 (char *prefix, _Decimal128 val128, _Decimal128 shouldbe)
 
   return (rc);
 }
-
+#endif
 void
 print_int128 (char *prefix, __int128 val128)
 {
@@ -342,11 +342,19 @@ void
 print_ibm128 (char *prefix, long double ldbl)
 {
   double high, low;
+  __VEC_U_128 t;
+
+  t.ldbl128 = ldbl;
+#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+  high = t.vf2 [1];
+  low = t.vf2 [0];
+#else
+  high = t.vf2 [0];
+  low = t.vf2 [1];
+#endif
 
   printf ("%s:% .34" "Le" " % .28" "La" "\n", prefix, ldbl, ldbl);
 
-  high = __builtin_unpack_longdouble (ldbl, 0);
-  low = __builtin_unpack_longdouble (ldbl, 1);
   printf ("   %s:% .14" "a" " % .14" "a" "\n", prefix, high, low);
 }
 
@@ -354,8 +362,17 @@ void
 print_ibm128x (char *prefix, long double ldbl)
 {
   double high, low;
-  high = __builtin_unpack_longdouble (ldbl, 0);
-  low = __builtin_unpack_longdouble (ldbl, 1);
+  __VEC_U_128 t;
+
+  t.ldbl128 = ldbl;
+#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+  high = t.vf2 [1];
+  low = t.vf2 [0];
+#else
+  high = t.vf2 [0];
+  low = t.vf2 [1];
+#endif
+
   printf ("%s:% .28La, /* %Lg */\n", prefix, ldbl, ldbl);
   printf ("\t:% .13a, % .13a*/\n", high, low);
 }
@@ -594,15 +611,18 @@ print_v2b64c (char *prefix, vb64_t val)
 #endif
 }
 
+typedef unsigned long long int ullint_t;
+typedef long long int llint_t;
+
 void
 print_v2b64x (char *prefix, vb64_t boolval)
 {
   vui64_t val = (vui64_t)boolval;
 
 #if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-  printf ("%s %016lx,%016lx\n", prefix, val[1], val[0]);
+  printf ("%s %016llx,%016llx\n", prefix, (ullint_t)val[1], (ullint_t)val[0]);
 #else
-  printf ("%s %016lx,%016lx\n", prefix, val[0], val[1]);
+  printf ("%s %016llx,%016llx\n", prefix, (ullint_t)val[0], (ullint_t)val[1]);
 #endif
 }
 
@@ -612,9 +632,9 @@ print_v2int64 (char *prefix, vui64_t val128)
   vui64_t val = (vui64_t) val128;
 
 #if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-  printf ("%s %ld,%ld\n", prefix, val[1], val[0]);
+  printf ("%s %lld,%lld\n", prefix, (llint_t)val[1], (llint_t)val[0]);
 #else
-  printf ("%s %ld,%ld\n", prefix, val[0], val[1]);
+  printf ("%s %lld,%lld\n", prefix, (llint_t)val[0], (llint_t)val[1]);
 #endif
 }
 
@@ -624,9 +644,9 @@ print_v2xint64 (char *prefix, vui64_t val128)
   vui64_t val = (vui64_t) val128;
 
 #if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-  printf ("%s %016lx,%016lx\n", prefix, val[1], val[0]);
+  printf ("%s %016llx,%016llx\n", prefix, (ullint_t)val[1], (ullint_t)val[0]);
 #else
-  printf ("%s %016lx,%016lx\n", prefix, val[0], val[1]);
+  printf ("%s %016llx,%016llx\n", prefix, (ullint_t)val[0], (ullint_t)val[1]);
 #endif
 }
 
@@ -677,6 +697,41 @@ print_vint256 (char *prefix, vui128_t val0_128, vui128_t val1_128)
 #endif
 }
 
+void
+print_vuint512x (char *prefix, vui128_t val0_128, vui128_t val1_128, vui128_t val2_128, vui128_t val3_128)
+{
+  vui32_t val0 = (vui32_t) val0_128;
+  vui32_t val1 = (vui32_t) val1_128;
+  vui32_t val2 = (vui32_t) val2_128;
+  vui32_t val3 = (vui32_t) val3_128;
+  int i;
+  int len = strlen (prefix);
+  char pad[64];
+
+  len = (len < 64) ? len : 63;
+  for (i=0; i < len; i++)
+    pad [i] = ' ';
+
+  pad [len] = 0;
+
+
+#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+  printf ("%s%08x%08x%08x%08x %08x%08x%08x%08x\n", prefix,
+	  val0[3], val0[2], val0[1], val0[0],
+	  val1[3], val1[2], val1[1], val1[0]);
+  printf ("%s%08x%08x%08x%08x %08x%08x%08x%08x\n", pad,
+	  val2[3], val2[2], val2[1], val2[0],
+	  val3[3], val3[2], val3[1], val3[0]);
+#else
+  printf ("%s%08x%08x%08x%08x %08x%08x%08x%08x\n", prefix,
+	  val0[0], val0[1], val0[2], val0[3],
+	  val1[0], val1[1], val1[2], val1[3]);
+  printf ("%s%08x%08x%08x%08x %08x%08x%08x%08x\n", pad,
+	  val2[3], val2[2], val2[1], val2[0],
+	  val3[3], val3[2], val3[1], val3[0]);
+#endif
+}
+
 void
 print_vb128c (char *prefix, vb128_t val)
 {
@@ -768,6 +823,88 @@ print_vint128_prod (char *prefix, vui32_t r, vui32_t a, vui32_t b, vui32_t c)
   print_vint128 (" *l = ", (vui128_t) r);
 }
 
+void
+print_vint512x (char *prefix, __VEC_U_512 r)
+{
+  printf ("vec %s\n", prefix);
+
+  vui32_t val0 = (vui32_t) r.vx3;
+  vui32_t val1 = (vui32_t) r.vx2;
+  vui32_t val2 = (vui32_t) r.vx1;
+  vui32_t val3 = (vui32_t) r.vx0;
+  int i;
+  int len = strlen (prefix);
+  char pad[64];
+
+  len = (len < 64) ? len : 63;
+  for (i=0; i < len; i++)
+    pad [i] = ' ';
+
+  pad [len] = 0;
+
+
+#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+  printf ("%s%08x%08x%08x%08x %08x%08x%08x%08x\n", prefix,
+	  val0[3], val0[2], val0[1], val0[0],
+	  val1[3], val1[2], val1[1], val1[0]);
+  printf ("%s%08x%08x%08x%08x %08x%08x%08x%08x\n", pad,
+	  val2[3], val2[2], val2[1], val2[0],
+	  val3[3], val3[2], val3[1], val3[0]);
+#else
+  printf ("%s%08x%08x%08x%08x %08x%08x%08x%08x\n", prefix,
+	  val0[0], val0[1], val0[2], val0[3],
+	  val1[0], val1[1], val1[2], val1[3]);
+  printf ("%s%08x%08x%08x%08x %08x%08x%08x%08x\n", pad,
+	  val2[3], val2[2], val2[1], val2[0],
+	  val3[3], val3[2], val3[1], val3[0]);
+#endif
+}
+
+void
+print_vint640x (char *prefix, __VEC_U_640 r)
+{
+  printf ("vec %s\n", prefix);
+  print_vint128x ("  =rh ", (vui128_t) r.vx4);
+  print_vint128x ("      ", (vui128_t) r.vx3);
+  print_vint128x ("      ", (vui128_t) r.vx2);
+  print_vint128x ("      ", (vui128_t) r.vx1);
+  print_vint128x ("   rl ", (vui128_t) r.vx0);
+}
+
+void
+print_vint512x_prod (char *prefix, __VEC_U_512 r, __VEC_U_256 a, __VEC_U_256 b)
+{
+  printf ("vec %s\n", prefix);
+  print_vint128x ("  a   ", (vui128_t) a.vx1);
+  print_vint128x ("      ", (vui128_t) a.vx0);
+  print_vint128x (" *b   ", (vui128_t) b.vx1);
+  print_vint128x ("      ", (vui128_t) b.vx0);
+  print_vint128x ("  =rh ", (vui128_t) r.vx3);
+  print_vint128x ("      ", (vui128_t) r.vx2);
+  print_vint128x ("      ", (vui128_t) r.vx1);
+  print_vint128x ("   rl ", (vui128_t) r.vx0);
+}
+
+void
+print_vint256x_prod (char *prefix, __VEC_U_256 r, vui128_t a, vui128_t b)
+{
+  printf ("vec %s\n", prefix);
+  print_vint128x ("  a * ", (vui128_t) a);
+  print_vint128x ("  b = ", (vui128_t) b);
+  print_vint128x (" *rh= ", (vui128_t) r.vx1);
+  print_vint128x (" *rl= ", (vui128_t) r.vx0);
+}
+
+void
+print_vint256_prod (char *prefix, __VEC_U_256 r, vui128_t a, vui128_t b)
+{
+  printf ("vec %s\n", prefix);
+  print_vint128 ("  a * ", (vui128_t) a);
+  print_vint128 ("  b = ", (vui128_t) b);
+  print_vint128 (" *rh= ", (vui128_t) r.vx1);
+  print_vint128 (" *rl= ", (vui128_t) r.vx0);
+}
+
 void
 print_vint128_extend (char *prefix, vui32_t r, vui32_t co, vui32_t a, vui32_t b,
                       vui32_t c)
@@ -793,7 +930,7 @@ print_vint128x_extend (char *prefix, vui32_t r, vui32_t co, vui32_t a,
 }
 
 void
-print_vfloat128x (char *prefix, __float128 val128)
+print_vfloat128x (char *prefix, __binary128 val128)
 {
   vui32_t val;
   vui16_t vh;
@@ -868,7 +1005,7 @@ print_vfloat128x (char *prefix, __float128 val128)
 }
 
 int
-check_isf128_priv (char *prefix, __float128 val128, int val, int shouldbe)
+check_isf128_priv (char *prefix, __binary128 val128, int val, int shouldbe)
 {
   int rc = 0;
   if (val != shouldbe)
@@ -937,6 +1074,26 @@ check_vint384_priv (char *prefix, vui128_t val0_128, vui128_t val1_128,
   return (rc);
 }
 
+int
+check_vint512_priv (char *prefix, __VEC_U_512 val_is, __VEC_U_512 val_sb)
+{
+  int rc = 0;
+  if (vec_any_ne ((vui32_t) val_is.vx0, (vui32_t) val_sb.vx0)
+   || vec_any_ne ((vui32_t) val_is.vx1, (vui32_t) val_sb.vx1)
+   || vec_any_ne ((vui32_t) val_is.vx2, (vui32_t) val_sb.vx2)
+   || vec_any_ne ((vui32_t) val_is.vx3, (vui32_t) val_sb.vx3))
+    {
+      rc = 1;
+      printf ("%s\n", prefix);
+      print_vuint512x ("    should be: ", val_sb.vx3, val_sb.vx2, val_sb.vx1,
+		       val_sb.vx0);
+      print_vuint512x ("           is: ", val_is.vx3, val_is.vx2, val_is.vx1,
+		       val_is.vx0);
+    }
+
+  return (rc);
+}
+
 int
 check_vui8_priv (char *prefix, vui8_t val128, vui8_t shouldbe)
 {
@@ -1209,6 +1366,9 @@ check_int64 (char *prefix, long val64, long shouldbe)
 
   return (rc);
 }
+
+#ifndef PVECLIB_DISABLE_DFP
+
 // TODO make this hidden so we can use TOC relative
 extern const _Decimal128 decpowof2[];
 
@@ -1230,22 +1390,6 @@ print_dfp128p2 (char *prefix, _Decimal128 val128, long exp)
   printf ("%s % .33DDe\n", prefix, x);
 }
 
-int
-check_f128bool_priv (char *prefix, __float128 val128, vb128_t boolis,
-                     vb128_t shouldbe)
-{
-  int rc = 0;
-  if (!vec_all_eq ((vui32_t)boolis, (vui32_t)shouldbe))
-    {
-      rc = 1;
-      print_vfloat128x (prefix, val128);
-      print_vint128x ("\tshould be: ", (vui128_t) shouldbe);
-      print_vint128x ("\t       is: ", (vui128_t) boolis);
-    }
-
-  return (rc);
-}
-
 int
 check_frexptftd_priv (char *prefix, _Decimal128 val128, long exp,
                       _Decimal128 shouldbe, long shouldexp)
@@ -1265,6 +1409,23 @@ check_frexptftd_priv (char *prefix, _Decimal128 val128, long exp,
 
   return (rc);
 }
+#endif
+
+int
+check_f128bool_priv (char *prefix, __binary128 val128, vb128_t boolis,
+                     vb128_t shouldbe)
+{
+  int rc = 0;
+  if (!vec_all_eq ((vui32_t)boolis, (vui32_t)shouldbe))
+    {
+      rc = 1;
+      print_vfloat128x (prefix, val128);
+      print_vint128x ("\tshould be: ", (vui128_t) shouldbe);
+      print_vint128x ("\t       is: ", (vui128_t) boolis);
+    }
+
+  return (rc);
+}
 
 int
 check_frexptfkf_priv (char *prefix, vui128_t val128, long exp,
