@@ -2458,7 +2458,7 @@ test_double_iszero (void)
 //#undef __DEBUG_PRINT__
 
 
-#define TIME_10_ITERATION 10
+#define TIMING_ITERATIONS 10
 #if 1
 int
 test_time_f64 (void)
@@ -2470,7 +2470,7 @@ test_time_f64 (void)
 
   printf ("\n%s is_f64 start, ...\n", __FUNCTION__);
   t_start = __builtin_ppc_get_timebase ();
-  for (i = 0; i < TIME_10_ITERATION; i++)
+  for (i = 0; i < TIMING_ITERATIONS; i++)
     {
       rc += timed_is_f64 ();
     }
@@ -2484,7 +2484,7 @@ test_time_f64 (void)
 
   printf ("\n%s fpclassify_f64 start, ...\n", __FUNCTION__);
   t_start = __builtin_ppc_get_timebase ();
-  for (i = 0; i < TIME_10_ITERATION; i++)
+  for (i = 0; i < TIMING_ITERATIONS; i++)
     {
       rc += timed_fpclassify_f64 ();
     }
