@@ -23,6 +23,8 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <pveclib/vec_int128_ppc.h>
+#include "arith128.h"
+#include <testsuite/arith128_print.h>
 
 vui128_t
 __test_msumudm (vui64_t a, vui64_t b, vui128_t c)
@@ -653,7 +655,15 @@ test_vec_mul10uq_c (vui128_t *p, vui128_t a)
   return vec_mul10uq (a);
 }
 
+/* This does not work because the target is applied after the CPP
+ * platform specific have been applied based on the command line
+ * =mcpu=<target>.
+ * Keep this example as a warning. */
 vui128_t
+#ifndef __clang__
+// clang does not either, generates a warning
+__attribute__((__target__("cpu=power9")))
+#endif
 test_vec_mul10uq (vui128_t a)
 {
 	return vec_mul10uq (a);
@@ -686,10 +696,17 @@ test_vec_revbq (vui128_t a)
 void
 test_vec_load_store (vui128_t *a, vui128_t *b)
   {
+#ifdef __clang__
+    vui32_t temp;
+
+    temp = vec_ld (0, (vui32_t *)a);
+    vec_st (temp, 0, (vui32_t *)b);
+#else
     vui64_t temp;
 
     temp = vec_ld (0, (vui64_t *)a);
     vec_st (temp, 0, (vui64_t *)b);
+#endif
   }
 
 vui64_t
@@ -809,12 +826,71 @@ __test_vmuleud (vui64_t a, vui64_t b)
   return (vec_vmuleud (a, b));
 }
 
+vui128_t
+__test_vmsumoud (vui64_t a, vui64_t b, vui128_t c)
+{
+  return vec_vmsumoud (a, b, c);
+}
+
+vui128_t
+__test_vmsumeud (vui64_t a, vui64_t b, vui128_t c)
+{
+  return vec_vmsumeud (a, b, c);
+}
+
+vui128_t
+__test_vmaddoud (vui64_t a, vui64_t b, vui64_t c)
+{
+  return vec_vmaddoud (a, b, c);
+}
+
+vui128_t
+__test_vmadd2oud (vui64_t a, vui64_t b, vui64_t c, vui64_t d)
+{
+  return vec_vmadd2oud (a, b, c, d);
+}
+
+vui128_t
+__test_vmaddeud (vui64_t a, vui64_t b, vui64_t c)
+{
+  return vec_vmaddeud (a, b, c);
+}
+
+vui128_t
+__test_vmadd2eud (vui64_t a, vui64_t b, vui64_t c, vui64_t d)
+{
+  return vec_vmadd2eud (a, b, c, d);
+}
+
 vui128_t
 __test_muludq (vui128_t *mulu, vui128_t a, vui128_t b)
 {
   return (vec_muludq (mulu, a, b));
 }
 
+vui128_t
+__test_madduq (vui128_t *mulu, vui128_t a, vui128_t b, vui128_t c)
+{
+  return vec_madduq (mulu, a, b, c);
+}
+
+vui128_t
+__test_madd2uq (vui128_t *mulu, vui128_t a, vui128_t b, vui128_t c, vui128_t d)
+{
+  return vec_madd2uq (mulu, a, b, c, d);
+}
+
+vui128_t
+__test_madduq_x (vui128_t *mulu, vui128_t a, vui128_t b, vui128_t c)
+{
+  vui128_t ph, pl, cl;
+  pl = vec_muludq (&ph, a, b);
+  pl = vec_adduqm (pl, c);
+  cl = vec_addcuq (pl, c);
+  *mulu = vec_adduqm (ph, cl);
+  return (pl);
+}
+
 vui128_t
 __test_mulluq (vui128_t a, vui128_t b)
 {
@@ -830,8 +906,8 @@ __test_mulhuq (vui128_t a, vui128_t b)
 vui128_t
 __test_mulhuq2 (vui128_t a, vui128_t b)
 {
-  vui128_t mq, r;
-  r = vec_muludq (&mq, a, b);
+  vui128_t mq;
+  vec_muludq (&mq, a, b);
   return mq;
 }
 
@@ -905,18 +981,18 @@ void
 test_mul4uq (vui128_t *__restrict__ mulu, vui128_t m1h, vui128_t m1l,
 	     vui128_t m2h, vui128_t m2l)
 {
-  vui128_t mc, mp, mq;
+  vui128_t mc, mp, mq, mqhl;
   vui128_t mphh, mphl, mplh, mpll;
   mpll = vec_muludq (&mplh, m1l, m2l);
   mp = vec_muludq (&mphl, m1h, m2l);
   mplh = vec_addcq (&mc, mplh, mp);
-  mphl = vec_addcuq (mphl, mc);
-  mp = vec_muludq (&mc, m2h, m1l);
+  mphl = vec_adduqm (mphl, mc);
+  mp = vec_muludq (&mqhl, m2h, m1l);
   mplh = vec_addcq (&mq, mplh, mp);
-  mphl = vec_addcq (&mc, mphl, mq);
+  mphl = vec_addeq (&mc, mphl, mqhl, mq);
   mp = vec_muludq (&mphh, m2h, m1h);
-  mplh = vec_addcq (&mc, mplh, mp);
-  mphl = vec_addcuq (mphh, mc);
+  mphl = vec_addcq (&mq, mphl, mp);
+  mphh = vec_addeuqm (mphh, mq, mc);
 
   mulu[0] = mpll;
   mulu[1] = mplh;
@@ -924,8 +1000,74 @@ test_mul4uq (vui128_t *__restrict__ mulu, vui128_t m1h, vui128_t m1l,
   mulu[3] = mphh;
 }
 
+#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+#define __NDX(__index) (__index)
+#define __MDX(__index) (__index)
+#define __PDX(__index) (__index)
+#else
+#define __NDX(__index) ((nx - 1) - (__index))
+#define __MDX(__index) ((mx - 1) - (__index))
+#define __PDX(__index) ((px - 1) - (__index))
+#endif
 void
-example_qw_convert_decimal (vui64_t *ten_16, vui128_t value)
+test_mul128_MN (vui128_t *p, vui128_t *m1, vui128_t *m2,
+		  unsigned long M, unsigned long N)
+{
+  vui128_t *mp = m1;
+  vui128_t *np = m2;
+  unsigned long mx = M;
+  unsigned long nx = N;
+#if __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
+  unsigned long px = M + N;
+#endif
+  unsigned long i, j;
+  vui128_t mpx0, mqx0, mpx1, mqx1;
+
+  /* sizeof(m1) < sizeof(m2) swap the pointers and quadword counts.
+   * This allows for early exit when size of either is 1. */
+  if (mx < nx)
+    {
+      vui128_t *xp;
+      unsigned long x;
+
+      x = mx;
+      xp = mp;
+      mx = nx;
+      mp = np;
+      nx = x;
+      np = xp;
+    }
+
+//  printf ("MXD(0)=%lu, NDX(0)=%lu, PDX(0)=%lu\n", __MDX(0), __NDX(0), __PDX(0));
+  mpx0 = vec_muludq (&mqx0, mp[__MDX(0)], np[__NDX(0)]);
+  p[__PDX(0)] = mpx0;
+  for ( i=1; i < mx; i++ )
+    {
+      mpx1 = vec_madduq (&mqx1, mp[__MDX(i)], np[__NDX(0)], mqx0);
+      p[__PDX(i)] = mpx1;
+      mqx0 = mqx1;
+//      printf ("MXD(i)=%lu, NDX(0)=%lu, PDX(i)=%lu\n", __MDX(i), __NDX(0), __PDX(i));
+    }
+  p[__PDX(mx)] = mqx0;
+//  printf ("PDX(mx)=%lu\n", __PDX(mx));
+
+  for  ( j=1; j < nx; j++ )
+    {
+      mpx0 = vec_madduq (&mqx0, mp[__MDX(0)], np[__NDX(j)], p[__PDX(j)]);
+      p[__PDX(j)] = mpx0;
+      for ( i=1; i < mx; i++ )
+        {
+          mpx1 = vec_madd2uq (&mqx1, mp[__MDX(i)], np[__NDX(j)], mqx0, p[__PDX(i+j)]);
+          p[__PDX(i+j)] = mpx1;
+          mqx0 = mqx1;
+        }
+      p[__PDX(mx+j)] = mqx0;
+    }
+
+}
+
+void
+example_qw_convert_decimal (uint64_t *ten_16, vui128_t value)
 {
   /* Magic numbers for multiplicative inverse to divide by 10**32
    are 211857340822306639531405861550393824741, corrective add,
@@ -946,6 +1088,7 @@ example_qw_convert_decimal (vui64_t *ten_16, vui128_t value)
 							 10000000000000000UL);
 
   vui128_t tmpq, tmpr, tmpc, tmp;
+  __VEC_U_128 tmp_16;
 
   // First divide/modulo by 10**32 to separate the top 7 digits from
   // the lower 32 digits
@@ -962,7 +1105,8 @@ example_qw_convert_decimal (vui64_t *ten_16, vui128_t value)
   tmpr = vec_subuqm (value, tmp);
 
   // return top 16 digits
-  ten_16[0] = (vui64_t) tmpq[VEC_DW_L];
+  tmp_16.vx1 = tmpq;
+  ten_16[0] = tmp_16.ulong.lower;
 
   // Next divide/modulo the remaining 32 digits by 10**16.
   // This separates the middle and low 16 digits into doublewords.
@@ -976,9 +1120,11 @@ example_qw_convert_decimal (vui64_t *ten_16, vui128_t value)
   tmpr = vec_subuqm (value, tmp);
 
   // return middle 16 digits
-  ten_16[1] = (vui64_t) tmpq[VEC_DW_L];
+  tmp_16.vx1 = tmpq;
+  ten_16[1] = tmp_16.ulong.lower;
   // return low 16 digits
-  ten_16[2] = (vui64_t) tmpr[VEC_DW_L];
+  tmp_16.vx1 = tmpr;
+  ten_16[2] = tmp_16.ulong.lower;
 }
 
 
@@ -1033,8 +1179,14 @@ example_print_vint128 (vi128_t value)
   tmp = vec_vmuloud (t_high, (vui64_t) mul_ten16);
   t_mid = (vui64_t) vec_subuqm (val128, tmp);
 
-  printf ("%c%07ld%016ld%016ld", sign, t_high[VEC_DW_L],
+  // Work-around for GCC PR 96139
+#ifdef __clang__
+  printf ("%c%07llu%016llu%016llu", sign, t_high[VEC_DW_L],
 	  t_mid[VEC_DW_L], t_low[VEC_DW_L]);
+#else
+  printf ("%c%07lu%016lu%016lu", sign, t_high[VEC_DW_L],
+	  t_mid[VEC_DW_L], t_low[VEC_DW_L]);
+#endif
 }
 
 void
@@ -1240,7 +1392,7 @@ __test_clzq (vui128_t vra)
 	const __vector unsigned long long vzero = {0,0};
 	const __vector unsigned long long v64 = {64, 64};
 
-	vt1 = vec_vclz ((__vector unsigned long long)vra);
+	vt1 = vec_clzd ((__vector unsigned long long)vra);
 	vt2 = (__vector unsigned long long)vec_cmpeq (vt1, v64);
 	vt3 = (__vector unsigned long long)vec_sld ((__vector unsigned char)vzero, (__vector unsigned char)vt2, 8);
 	result = vec_andc(vt1, vt3);
