@@ -21,14 +21,20 @@
  */
 
 //#pragma GCC push target
-#pragma GCC target ("cpu=power9")
+#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+//#pragma GCC target ("cpu=power9")
 #include <stdint.h>
 #include <stdio.h>
 #include <fenv.h>
 #include <float.h>
+#ifndef PVECLIB_DISABLE_F128MATH
+/* Disable for __clang__ because of bug involving <math.h>
+   incombination with -mcpu=power9 -mfloat128 */
 #include <math.h>
+#endif
 
 #include <pveclib/vec_int128_ppc.h>
+#include <pveclib/vec_int512_ppc.h>
 #include <pveclib/vec_f128_ppc.h>
 #include <pveclib/vec_f32_ppc.h>
 #include <pveclib/vec_bcd_ppc.h>
@@ -87,6 +93,98 @@ __test_vmuleud_PWR9 (vui64_t a, vui64_t b)
   return vec_vmuleud (a, b);
 }
 
+vui128_t
+__test_vmsumoud_PWR9 (vui64_t a, vui64_t b, vui128_t c)
+{
+  return vec_vmsumoud (a, b, c);
+}
+
+vui128_t
+__test_vmsumeud_PWR9 (vui64_t a, vui64_t b, vui128_t c)
+{
+  return vec_vmsumeud (a, b, c);
+}
+
+vui128_t
+__test_vmaddoud_PWR9 (vui64_t a, vui64_t b, vui64_t c)
+{
+  return vec_vmaddoud (a, b, c);
+}
+
+vui128_t
+__test_vmaddeud_PWR9 (vui64_t a, vui64_t b, vui64_t c)
+{
+  return vec_vmaddeud (a, b, c);
+}
+
+vui128_t
+__test_vmadd2oud_PWR9 (vui64_t a, vui64_t b, vui64_t c, vui64_t d)
+{
+  return vec_vmadd2oud (a, b, c, d);
+}
+
+vui128_t
+__test_vmadd2oud_x_PWR9 (vui64_t a, vui64_t b, vui64_t c, vui64_t d)
+{
+  /* msumudm provides 2 x DW muliplies and 3 way 128-bit sum of the
+   * two products and the addend c.
+   * We want the 3 way 128-bit sum of one DW multiply and two DW
+   * addends c and d ((A.odd * B.odd) + D.odd + C.odd).
+   * So we convert the inputs to compute:
+   * (A.odd * B.odd) + (1 * D.odd) + C.odd which executes
+   * in the single msumudm.
+   */
+  const vui64_t one =  CONST_VINT64_DW ( 0, 1 );
+  vui64_t admrg = vec_mrgald ((vui128_t) d, (vui128_t) a);
+  vui64_t b1mrg = vec_mrgald ((vui128_t) one, (vui128_t) b);
+  /* The high (even) DW of one is zero so only 1 QW const is needed.
+   * Effectively vec_mrgald ((vui128_t) zero, (vui128_t) c). */
+  vui64_t c_oud = vec_pasted ( one, c);
+  return vec_msumudm(admrg, b1mrg, (vui128_t) c_oud);
+}
+
+vui128_t
+__test_vmadd2eud_PWR9 (vui64_t a, vui64_t b, vui64_t c, vui64_t d)
+{
+  return vec_vmadd2eud (a, b, c, d);
+}
+
+vui128_t
+__test_vmadd2eud_x_PWR9 (vui64_t a, vui64_t b, vui64_t c, vui64_t d)
+{
+  /* msumudm provides 2 x DW muliplies and 3 way 128-bit sum of the
+   * two products and the addend c.
+   * We want the 3 way 128-bit sum of one DW multiply and two DW
+   * addends c and d ((A.even * B,even) + D.even + C.even).
+   * So we convert the inputs to compute:
+   * (A.even * B.even) + (D.even * 1) + C.even which executes
+   * in the single msumudm.
+   */
+  const vui64_t one =  CONST_VINT64_DW ( 0, 1 );
+  vui64_t admrg = vec_mrgahd ((vui128_t) a, (vui128_t) d);
+  vui64_t b1mrg = vec_pasted ( b, one);
+  vui64_t c_eud = vec_mrgahd ((vui128_t) one, (vui128_t) c);
+  return vec_msumudm(admrg, b1mrg, (vui128_t) c_eud);
+}
+
+vui128_t
+__test_vmadd2ud_PWR9 (vui64_t a, vui64_t b, vui64_t c, vui64_t d)
+{
+  vui128_t e, o;
+  e = vec_vmadd2eud (a, b, c, d);
+  o = vec_vmadd2oud (a, b, c, d);
+  return vec_adduqm (e, o);
+}
+
+vui128_t
+__test_vmadd2ud_x_PWR9 (vui64_t a, vui64_t b, vui64_t c, vui64_t d)
+{
+  vui128_t e, o;
+  e = __test_vmadd2eud_x_PWR9 (a, b, c, d);
+  o = __test_vmadd2oud_x_PWR9 (a, b, c, d);
+  return vec_adduqm (e, o);
+}
+
 vui128_t
 __test_mulhuq_PWR9 (vui128_t a, vui128_t b)
 {
@@ -105,6 +203,199 @@ __test_muludq_PWR9 (vui128_t *mulh, vui128_t a, vui128_t b)
   return vec_muludq (mulh, a, b);
 }
 
+vui128_t
+__test_muludq_x_PWR9 (vui128_t *mulu, vui128_t a, vui128_t b)
+{
+  /* compute the 256 bit product of two 128 bit values a, b.
+   * The high 128 bits are accumulated in thq and the low 128-bits
+   * in tlq. The high 128-bits of the product are returned to the
+   * address of the 1st parm. The low 128-bits are the return
+   * value.
+   */
+  const vui64_t zero = { 0, 0 };
+  vui64_t a_swap = vec_swapd ((vui64_t) a);
+  vui128_t thq, tlq, tx;
+  vui128_t t0l, tc1;
+  vui128_t thh, thl, tlh, tll;
+  /* multiply the low 64-bits of a and b.  For PWR9 this is just
+   * vmsumudm with conditioned inputs.  */
+  tll = vec_vmuloud ((vui64_t)a, (vui64_t)b);
+  thh = vec_vmuleud ((vui64_t)a, (vui64_t)b);
+  thl = vec_vmuloud (a_swap, (vui64_t)b);
+  tlh = vec_vmuleud (a_swap, (vui64_t)b);
+  /* sum the two middle products (plus the high 64-bits of the low
+   * product).  This will generate a carry that we need to capture.  */
+  t0l   = (vui128_t) vec_mrgahd ( (vui128_t) zero, tll);
+  tc1 = vec_addcuq (thl, tlh);
+  tx   = vec_adduqm (thl, tlh);
+  tx   = vec_adduqm (tx, t0l);
+  /* result = t[l] || tll[l].  */
+  tlq = (vui128_t) vec_mrgald ((vui128_t) tx, (vui128_t) tll);
+  /* Sum the high product plus the high sum (with carry) of middle
+   * partial products.  This can't overflow.  */
+  thq = (vui128_t) vec_permdi ((vui64_t) tc1, (vui64_t) tx, 2);
+  thq = vec_adduqm ( thh, thq);
+
+  *mulu = (vui128_t) thq;
+  return ((vui128_t) tlq);
+}
+
+vui128_t
+__test_muludq_y_PWR9 (vui128_t *mulu, vui128_t a, vui128_t b)
+{
+  vui32_t t, tmq;
+  /* compute the 256 bit product of two 128 bit values a, b.
+   * The high 128 bits are accumulated in t and the low 128-bits
+   * in tmq. The high 128-bits of the product are returned to the
+   * address of the 1st parm. The low 128-bits are the return
+   * value.
+   */
+  vui64_t a_swap = vec_swapd ((vui64_t) a);
+  vui128_t tab, tba, tc1;
+  /* multiply the low 64-bits of a and b.  For PWR9 this is just
+   * vmsumudm with conditioned inputs.  */
+  tmq = (vui32_t) vec_vmuloud ((vui64_t)a, (vui64_t)b);
+  /* compute the 2 middle partial projects.  Use vmaddeud to add the
+   * high 64-bits of the low product to one of the middle products.
+   * This can not overflow.  */
+  tab = vec_vmuloud (a_swap, (vui64_t) b);
+  tba = vec_vmaddeud (a_swap, (vui64_t) b, (vui64_t) tmq);
+  /* sum the two middle products (plus the high 64-bits of the low
+   * product.  This will generate a carry that we need to capture.  */
+  t   = (vui32_t) vec_adduqm (tab, tba);
+  tc1 = vec_addcuq (tab, tba);
+  /* result = t[l] || tmq[l].  */
+  tmq = (vui32_t) vec_mrgald ((vui128_t) t, (vui128_t) tmq);
+  /* we can use multiply sum here because the high product plus the
+   * high sum of middle partial products can't overflow.  */
+  t   = (vui32_t) vec_permdi ((vui64_t) tc1, (vui64_t) t, 2);
+  // This is equivalent to vec_vmadd2eud(a, b, tab, tba)
+  // were (tab_even + tba_even) was pre-computed including the carry,
+  // so no masking is required.
+  t   = (vui32_t) vec_vmsumeud ((vui64_t) a, (vui64_t) b, (vui128_t) t);
+
+  *mulu = (vui128_t) t;
+  return ((vui128_t) tmq);
+}
+
+vui128_t
+__test_muludq_z_PWR9 (vui128_t *mulu, vui128_t a, vui128_t b)
+{
+  vui32_t t, tmq;
+  /* compute the 256 bit product of two 128 bit values a, b.
+   * The high 128 bits are accumulated in t and the low 128-bits
+   * in tmq. The high 128-bits of the product are returned to the
+   * address of the 1st parm. The low 128-bits are the return
+   * value.
+   */
+  const vui64_t zero = { 0, 0 };
+  vui64_t b_splto = vec_spltd ((vui64_t) b, 1);
+  vui64_t b_splte = vec_spltd ((vui64_t) b, 0);
+  vui128_t tb0, tb1;
+  // multiply a by the odd 64-bits of b.  For PWR9 this is just
+  // vmsumudm with conditioned inputs.
+  tb1 = vec_vmuloud ((vui64_t)a, (vui64_t)b_splto);
+  tb0 = vec_vmuleud ((vui64_t)a, (vui64_t)b_splto);
+  // save the low dw of a_low * b_low in tmq
+  tmq = (vui32_t) tb1;
+  // shift a_odd * b_odd right 64-bits
+  tb1 = (vui128_t) vec_mrgahd ((vui128_t) zero, (vui128_t) tb1);
+  // Sum (a_even * b_odd) + ((a_odd * b_odd) >> 64))
+  t   = (vui32_t) vec_adduqm (tb0, tb1);
+
+  // multiply a by the even 64-bits of b sum with t.
+  // For PWR9 this is just vmsumudm with conditioned inputs.
+  tb1 = vec_vmaddoud ((vui64_t)a, (vui64_t)b_splte, (vui64_t)t);
+  tb0 = vec_vmaddeud ((vui64_t)a, (vui64_t)b_splte, (vui64_t)t);
+  // merge low dw of (a_odd * b_even + t_odd) with low dw of (a_low * b_low)
+  tmq = (vui32_t) vec_mrgald ((vui128_t) tb1, (vui128_t) tmq);
+  // shift a_odd * b_even + t_odd right 64-bits
+  tb1 = (vui128_t) vec_mrgahd ((vui128_t) zero, (vui128_t) tb1);
+  t   = (vui32_t) vec_adduqm (tb0, tb1);
+
+  *mulu = (vui128_t) t;
+  return ((vui128_t) tmq);
+}
+
+vui128_t
+__test_madduq_PWR9 (vui128_t *mulu, vui128_t a, vui128_t b, vui128_t c)
+{
+  return vec_madduq (mulu, a, b, c);
+}
+
+vui128_t
+__test_madduq_x_PWR9 (vui128_t *mulu, vui128_t a, vui128_t b, vui128_t c)
+{
+  // compute the 256 bit sum of product of two 128 bit values a, b
+  // plus the quadword addend c.
+  const vui64_t zero = { 0, 0 };
+  vui64_t a_swap = vec_swapd ((vui64_t) a);
+  vui128_t thq, tlq, tx;
+  vui128_t t0l, tc1;
+  vui128_t thh, thl, tlh, tll;
+  // multiply the low 64-bits of a and b.  For PWR9 this is just
+  // vmsumudm with conditioned inputs.
+  tll = vec_vmaddoud ((vui64_t)a, (vui64_t)b, (vui64_t)c);
+  thh = vec_vmuleud ((vui64_t)a, (vui64_t)b);
+  thl = vec_vmuloud (a_swap, (vui64_t)b);
+  tlh = vec_vmaddeud (a_swap, (vui64_t)b, (vui64_t)c);
+  // sum the two middle products (plus the high 64-bits of the low
+  // product).  This will generate a carry that we need to capture.
+  t0l = (vui128_t) vec_mrgahd ( (vui128_t) zero, tll);
+  tc1 = vec_addcuq (thl, tlh);
+  tx  = vec_adduqm (thl, tlh);
+  tx  = vec_adduqm (tx, t0l);
+  // result = t[l] || tll[l].
+  tlq = (vui128_t) vec_mrgald ((vui128_t) tx, (vui128_t) tll);
+  // Sum the high product plus the high sum (with carry) of middle
+  // partial products.  This can't overflow.
+  thq = (vui128_t) vec_permdi ((vui64_t) tc1, (vui64_t) tx, 2);
+  thq = vec_adduqm ( thh, thq);
+
+  *mulu = (vui128_t) thq;
+  return ((vui128_t) tlq);
+}
+
+vui128_t
+__test_madduq_y_PWR9 (vui128_t *mulu, vui128_t a, vui128_t b, vui128_t c)
+{
+  // compute the 256 bit sum of product of two 128 bit values a, b
+  // plus the quadword addend c.
+  vui64_t a_swap = vec_swapd ((vui64_t) a);
+  vui128_t thq, tlq, tx;
+  vui128_t t0l, tc1, tcl;
+  vui128_t thh, thl, tlh, tll;
+  // multiply the four combinations of a_odd/a_even by b_odd/b_even.
+  tll = vec_vmuloud ((vui64_t)a, (vui64_t)b);
+  thh = vec_vmuleud ((vui64_t)a, (vui64_t)b);
+  thl = vec_vmuloud (a_swap, (vui64_t)b);
+  tlh = vec_vmuleud (a_swap, (vui64_t)b);
+  /* Add c to lower 128-bits of the partial product. */
+  tcl = vec_addcuq (tll, c);
+  tll = vec_adduqm (tll, c);
+  t0l = (vui128_t) vec_permdi ((vui64_t) tcl, (vui64_t) tll, 2);
+  // sum the two middle products (plus the high 65-bits of the low
+  // product-sum.
+  tc1 = vec_addcuq (thl, tlh);
+  tx  = vec_adduqm (thl, tlh);
+  tx  = vec_adduqm (tx, t0l);
+  // result = tx[l]_odd || tll[l]_odd.
+  tlq = (vui128_t) vec_mrgald ((vui128_t) tx, (vui128_t) tll);
+  // Sum the high product plus the high sum (with carry) of middle
+  // partial products.  This can't overflow.
+  thq = (vui128_t) vec_permdi ((vui64_t) tc1, (vui64_t) tx, 2);
+  thq = vec_adduqm ( thh, thq);
+
+  *mulu = (vui128_t) thq;
+  return ((vui128_t) tlq);
+}
+
+vui128_t
+__test_madd2uq_PWR9 (vui128_t *mulu, vui128_t a, vui128_t b, vui128_t c, vui128_t d)
+{
+  return vec_madd2uq (mulu, a, b, c, d);
+}
+
 vui128_t
 __test_mulhluq_PWR9 (vui128_t *mulh, vui128_t a, vui128_t b)
 {
@@ -115,8 +406,8 @@ __test_mulhluq_PWR9 (vui128_t *mulh, vui128_t a, vui128_t b)
 vui128_t
 __test_mulhuq2_PWR9 (vui128_t a, vui128_t b)
 {
-  vui128_t mq, r;
-  r = vec_muludq (&mq, a, b);
+  vui128_t mq;
+  vec_muludq (&mq, a, b);
   return mq;
 }
 
@@ -314,6 +605,8 @@ test_vec_iszerof32_PWR9 (vf32_t value)
 {
   return (vec_iszerof32 (value));
 }
+
+#if (__BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__)
 int
 test_vec_all_finitef64_PWR9 (vf64_t value)
 {
@@ -420,6 +713,7 @@ test_vec_iszerof64_PWR9 (vf64_t value)
 {
   return (vec_iszerof64 (value));
 }
+#endif
 
 vb128_t
 test_vec_isfinitef128_PWR9 (__binary128 f128)
@@ -515,6 +809,7 @@ test_f32_zero_demorm_PWR9 (vf32_t value)
   return result;
 }
 
+#ifndef PVECLIB_DISABLE_F128MATH
 vui32_t
 test_fpclassify_f32_PWR9 (vf32_t value)
 {
@@ -557,6 +852,7 @@ test_fpclassify_f32loop_PWR9 (vui32_t *out, vf32_t *in, int count)
       out[i] = test_fpclassify_f32_PWR9 (in[i]);
     }
 }
+#endif
 
 /* dummy sinf32 example. From Posix:
  * If value is NaN then return a NaN.
@@ -883,6 +1179,7 @@ __test_vec_extract_sig_f32 (vf32_t val)
 }
 #endif
 
+#ifndef PVECLIB_DISABLE_DFP
 vBCD_t
 test_vec_bcdmulh_PWR9 (vBCD_t a, vBCD_t b)
 {
@@ -1152,6 +1449,7 @@ test_vec_bcdcfuq_PWR9 (vui128_t vra)
 {
   return vec_bcdcfuq (vra);
 }
+#endif
 
 vi128_t
 __test_remsq_10e31_PWR9  (vi128_t a, vi128_t b)
@@ -1206,7 +1504,7 @@ __test_remudq_10e32_PWR9  (vui128_t a, vui128_t b)
 }
 
 void
-example_qw_convert_decimal_PWR9 (vui64_t *ten_16, vui128_t value)
+example_qw_convert_decimal_PWR9 (uint64_t *ten_16, vui128_t value)
 {
   /* Magic numbers for multiplicative inverse to divide by 10**32
    are 211857340822306639531405861550393824741, corrective add,
@@ -1227,6 +1525,7 @@ example_qw_convert_decimal_PWR9 (vui64_t *ten_16, vui128_t value)
 							 10000000000000000UL);
 
   vui128_t tmpq, tmpr, tmpc, tmp;
+  __VEC_U_128 tmp_16;
 
   // First divide/modulo by 10**32 to separate the top 7 digits from
   // the lower 32 digits
@@ -1243,7 +1542,8 @@ example_qw_convert_decimal_PWR9 (vui64_t *ten_16, vui128_t value)
   tmpr = vec_subuqm (value, tmp);
 
   // return top 16 digits
-  ten_16[0] = (vui64_t) tmpq[VEC_DW_L];
+  tmp_16.vx1 = tmpq;
+  ten_16[0] = tmp_16.ulong.lower;
 
   // Next divide/modulo the remaining 32 digits by 10**16.
   // This separates the middle and low 16 digits into doublewords.
@@ -1257,9 +1557,11 @@ example_qw_convert_decimal_PWR9 (vui64_t *ten_16, vui128_t value)
   tmpr = vec_subuqm (value, tmp);
 
   // return middle 16 digits
-  ten_16[1] = (vui64_t) tmpq[VEC_DW_L];
+  tmp_16.vx1 = tmpq;
+  ten_16[1] = tmp_16.ulong.lower;
   // return low 16 digits
-  ten_16[2] = (vui64_t) tmpr[VEC_DW_L];
+  tmp_16.vx1 = tmpr;
+  ten_16[2] = tmp_16.ulong.lower;
 }
 
 vui128_t
@@ -1286,6 +1588,7 @@ example_longdiv_10e31_PWR9 (vui128_t *q, vui128_t *d, long int _N)
   return rh;
 }
 
+#ifndef PVECLIB_DISABLE_DFP
 long int
 example_longbcdct_10e32_PWR9 (vui128_t *d, vBCD_t decimal, long int _C , long int _N)
 {
@@ -1393,9 +1696,11 @@ example_longbcdct_10e31_PWR9 (vui128_t *d, vBCD_t decimal, long int _C,
 
   return cnt;
 }
+#endif
 
 void
-test_muluq_4x1_PWR9 (vui128_t *__restrict__ mulu, vui128_t m10, vui128_t m11, vui128_t m12, vui128_t m13, vui128_t m2)
+test_muluq_4x1_PWR9 (vui128_t *__restrict__ mulu, vui128_t m10, vui128_t m11,
+		     vui128_t m12, vui128_t m13, vui128_t m2)
 {
   vui128_t mq2, mq1, mq0, mq, mc;
   vui128_t mpx0, mpx1, mpx2, mpx3;
@@ -1406,29 +1711,31 @@ test_muluq_4x1_PWR9 (vui128_t *__restrict__ mulu, vui128_t m10, vui128_t m11, vu
   mpx1 = vec_addeq (&mc, mpx1, mq1, mc);
   mpx0 = vec_muludq (&mq, m10, m2);
   mpx0 = vec_addeq (&mc, mpx0, mq0, mc);
-  mq   = vec_adduqm (mc, mq);
+  mq = vec_adduqm (mc, mq);
 
-  mulu[0] = mpx0;
-  mulu[1] = mpx1;
-  mulu[2] = mpx2;
-  mulu[3] = mpx3;
+  mulu[0] = mq;
+  mulu[1] = mpx0;
+  mulu[2] = mpx1;
+  mulu[3] = mpx2;
+  mulu[4] = mpx3;
 }
 
 void
-test_mul4uq_PWR9 (vui128_t *__restrict__ mulu, vui128_t m1h, vui128_t m1l, vui128_t m2h, vui128_t m2l)
+test_mul4uq_PWR9 (vui128_t *__restrict__ mulu, vui128_t m1h, vui128_t m1l,
+		  vui128_t m2h, vui128_t m2l)
 {
-  vui128_t mc, mp, mq;
+  vui128_t mc, mp, mq, mqhl;
   vui128_t mphh, mphl, mplh, mpll;
   mpll = vec_muludq (&mplh, m1l, m2l);
   mp = vec_muludq (&mphl, m1h, m2l);
   mplh = vec_addcq (&mc, mplh, mp);
-  mphl = vec_addcuq (mphl, mc);
-  mp = vec_muludq (&mc, m2h, m1l);
+  mphl = vec_adduqm (mphl, mc);
+  mp = vec_muludq (&mqhl, m2h, m1l);
   mplh = vec_addcq (&mq, mplh, mp);
-  mphl = vec_addcq (&mc, mphl, mq);
+  mphl = vec_addeq (&mc, mphl, mqhl, mq);
   mp = vec_muludq (&mphh, m2h, m1h);
-  mplh = vec_addcq (&mc, mplh, mp);
-  mphl = vec_addcuq (mphh, mc);
+  mphl = vec_addcq (&mq, mphl, mp);
+  mphh = vec_addeuqm (mphh, mq, mc);
 
   mulu[0] = mpll;
   mulu[1] = mplh;
@@ -1436,4 +1743,139 @@ test_mul4uq_PWR9 (vui128_t *__restrict__ mulu, vui128_t m1h, vui128_t m1l, vui12
   mulu[3] = mphh;
 }
 
+__VEC_U_1024  __attribute__((flatten ))
+__test_madd512x512a512_PWR9 (__VEC_U_512 m1, __VEC_U_512 m2, __VEC_U_512 a1)
+{
+  __VEC_U_1024 result;
+#ifndef __clang__
+// Clang does not support register variables.
+  register vui128_t t0 asm("vs10");
+  register vui128_t t1 asm("vs11");
+  register vui128_t t2 asm("vs12");
+#else
+  vui128_t t0, t1, t2;
+#endif
+  __VEC_U_512x1 mp3, mp2, mp1, mp0;
+
+  mp0.x640 = vec_madd512x128a512_inline (m1, m2.vx0, a1);
+  t0 = mp0.x3.v1x128;
+  COMPILE_FENCE;
+  mp1.x640 = vec_madd512x128a512_inline (m1, m2.vx1, mp0.x3.v0x512);
+  t1 = mp1.x3.v1x128;
+  COMPILE_FENCE;
+  mp2.x640 = vec_madd512x128a512_inline (m1, m2.vx2, mp1.x3.v0x512);
+  t2 = mp2.x3.v1x128;
+  COMPILE_FENCE;
+  mp3.x640 = vec_madd512x128a512_inline (m1, m2.vx3, mp2.x3.v0x512);
+
+  result.vx0 = t0;
+  result.vx1 = t1;
+  result.vx2 = t2;
+  result.vx3 = mp3.x3.v1x128;
+  result.vx4 = mp3.x3.v0x512.vx0;
+  result.vx5 = mp3.x3.v0x512.vx1;
+  result.vx6 = mp3.x3.v0x512.vx2;
+  result.vx7 = mp3.x3.v0x512.vx3;
+  return result;
+}
+
+void
+test_mul128_MN_PWR9 (vui128_t *p, vui128_t *m1, vui128_t *m2, unsigned long M,
+		     unsigned long N)
+{
+  vui128_t *mp = m1;
+  vui128_t *np = m2;
+  unsigned long mx = M;
+  unsigned long nx = N;
+  unsigned long i, j;
+  vui128_t mpx0, mqx0, mpx1, mqx1;
+
+  /* sizeof(m1) < sizeof(m2) swap the pointers and quadword counts.
+   * This allows for early exit when size of either is 1. */
+  if (mx < nx)
+    {
+      vui128_t *xp;
+      unsigned long x;
+
+      x = mx;
+      xp = mp;
+      mx = nx;
+      mp = np;
+      nx = x;
+      np = xp;
+    }
+
+  mpx0 = vec_muludq (&mqx0, mp[0], np[0]);
+  p[0] = mpx0;
+  for (i = 1; i < mx; i++)
+    {
+      mpx1 = vec_madduq (&mqx1, mp[i], np[0], mqx0);
+      p[i] = mpx1;
+      mqx0 = mqx1;
+    }
+  p[mx] = mqx0;
+
+  for (j = 1; j < nx; j++)
+    {
+      mpx0 = vec_madduq (&mqx0, mp[0], np[j], p[j]);
+      p[j] = mpx0;
+      for (i = 1; i < mx; i++)
+	{
+	  mpx1 = vec_madd2uq (&mqx1, mp[i], np[j], mqx0, p[i + j]);
+	  p[i + j] = mpx1;
+	  mqx0 = mqx1;
+	}
+      p[mx + j] = mqx0;
+    }
+}
+
+void __attribute__((flatten ))
+#if 0
+test_vec_mul1024x1024_PWR9 (__VEC_U_2048* restrict r2048,
+				  __VEC_U_1024* restrict m1_1024,
+				  __VEC_U_1024* restrict m2_1024)
+#else
+test_vec_mul1024x1024_PWR9 (__VEC_U_2048* r2048,
+				  __VEC_U_1024* m1_1024,
+				  __VEC_U_1024* m2_1024)
+#endif
+{
+  __VEC_U_1024x512 subp0, subp1, subp2, subp3;
+  __VEC_U_512x1 sum1, sum2, sum3, sumx;
+  __VEC_U_512 temp[3];
+#if 0
+  __VEC_U_1024x512 * restrict pm1 = (__VEC_U_1024x512 *) m1_1024;
+  __VEC_U_1024x512 * restrict pm2 = (__VEC_U_1024x512 *) m2_1024;
+  __VEC_U_2048x512 * restrict pm2048 = (__VEC_U_2048x512 *) r2048;
+#else
+  __VEC_U_1024x512 * pm1 = (__VEC_U_1024x512 *) m1_1024;
+  __VEC_U_1024x512 * pm2 = (__VEC_U_1024x512 *) m2_1024;
+  __VEC_U_2048x512 * pm2048 = (__VEC_U_2048x512 *) r2048;
+#endif
+
+  subp0.x1024 = vec_mul512x512_inline (pm1->x2.v0x512, pm2->x2.v0x512);
+  pm2048->x4.v0x512 = subp0.x2.v0x512;
+
+  subp1.x1024 = vec_mul512x512_inline (pm1->x2.v1x512, pm2->x2.v0x512);
+  sum1.x640 = vec_add512cu (subp1.x2.v0x512, subp0.x2.v1x512);
+
+  temp[0] = sum1.x2.v0x512;
+  temp[1] = vec_add512ze (subp1.x2.v1x512, sum1.x2.v1x128);
+  COMPILE_FENCE;
+
+  subp2.x1024 = vec_mul512x512_inline (pm1->x2.v0x512, pm2->x2.v1x512);
+  sum2.x640 = vec_add512cu (temp[0], subp2.x2.v0x512);
+  temp[2] = sum2.x2.v0x512;
+  pm2048->x4.v1x512 = temp[2];
+  sumx.x640 = vec_add512ecu (temp[1], subp2.x2.v1x512, sum2.x2.v1x128);
+  temp[1] = sumx.x2.v0x512;
+  COMPILE_FENCE;
+
+  subp3.x1024 = vec_mul512x512_inline (pm1->x2.v1x512, pm2->x2.v1x512);
+  sum3.x640 = vec_add512cu (sumx.x2.v0x512, subp3.x2.v0x512);
+  pm2048->x4.v2x512 = sum3.x2.v0x512;
+  pm2048->x4.v3x512 = vec_add512ze2 (subp3.x2.v1x512, sumx.x2.v1x128,
+				     sum3.x2.v1x128);
+}
+#endif
 //#pragma GCC pop target
